/* 
 * File:   interrupt_config.h
 * Author: Chris St. George
 */

#ifndef INTERRUPT_CONFIG_H
#define	INTERRUPT_CONFIG_H

#include "config.h"

// Interrupt Flag, Priority, and Enable bits

#define Global_Interrupts_Enabled asm("ei")
#define Global_Interrupts_Disabled asm("di")
#define Multivector_Interrupts_Enable INTCONbits.MVEC

#define Timer_1_Vector _TIMER_1_VECTOR
#define Timer_1_Interrupt_Flag IFS0bits.T1IF
#define Timer_1_Interrupt_Priority IPC1bits.T1IP
#define Timer_1_Interrupt_Subpriority IPC1bits.T1IS
#define Timer_1_Interrupt_Enable IEC0bits.T1IE

#define Timer_2_Vector _TIMER_2_VECTOR
#define Timer_2_Interrupt_Flag IFS0bits.T2IF
#define Timer_2_Interrupt_Priority IPC2bits.T2IP
#define Timer_2_Interrupt_Subpriority IPC2bits.T2IS
#define Timer_2_Interrupt_Enable IEC0bits.T2IE

#define Timer_3_Vector _TIMER_3_VECTOR
#define Timer_3_Interrupt_Flag IFS0bits.T3IF
#define Timer_3_Interrupt_Priority IPC3bits.T3IP
#define Timer_3_Interrupt_Subpriority IPC3bits.T3IS
#define Timer_3_Interrupt_Enable IEC0bits.T3IE

#define Timer_4_Vector _TIMER_4_VECTOR
#define Timer_4_Interrupt_Flag IFS0bits.T4IF
#define Timer_4_Interrupt_Priority IPC4bits.T4IP
#define Timer_4_Interrupt_Subpriority IPC4bits.T4IS
#define Timer_4_Interrupt_Enable IEC0bits.T4IE

#define Timer_5_Vector _TIMER_5_VECTOR
#define Timer_5_Interrupt_Flag IFS0bits.T5IF
#define Timer_5_Interrupt_Priority IPC5bits.T5IP
#define Timer_5_Interrupt_Subpriority IPC5bits.T5IS
#define Timer_5_Interrupt_Enable IEC0bits.T5IE

#define PRAGMA(x) _Pragma(#x)
#define VECTOR(x) _##x##_VECTOR
#define _PRIORITY(x) ipl ## x ## auto
#define PRIORITY(x) _PRIORITY(x)

#define Interrupt_Settings(X) X##_Interrupt_Settings()
#define _Configure(X) Interrupt_Settings(X);
#define Configure(...)\
 void configure_interrupts() {\
  printf("\n\rvoid configure_interrupts()");\
  Global_Interrupts_Enabled;\
  Multivector_Interrupts_Enable = enabled;\
  FOR_EACH(_Configure,__VA_ARGS__)\
 }

#define Interrupt(object, full_name, event_type, priority, subpriority, is_enabled)\
 void Interrupt_Settings(object) {\
  /*interrupt object##_##event_type##_Event autoIPL(priority) vector VECTOR(object);*/\
  object##_Interrupt_Priority = priority; /* Sets the priority for the interrupt */\
  object##_Interrupt_Subpriority = subpriority; /* Sets the subpriority for the interrupt */\
  object##_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Interrupt_Enable = is_enabled; /* Sets the enable bit for the interrupt */\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()

/*<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>*/

// LED Timer (Timer 2)

#define PWM_Timer_Interrupt_Flag Timer_2_Interrupt_Flag
#define PWM_Timer_Interrupt_Priority Timer_2_Interrupt_Priority
#define PWM_Timer_Interrupt_Subpriority Timer_2_Interrupt_Subpriority
#define PWM_Timer_Interrupt_Enable Timer_2_Interrupt_Enable
#define PWM_Timer_Vector _TIMER_2_VECTOR

#define MEASURE_Timer_Interrupt_Flag Timer_4_Interrupt_Flag
#define MEASURE_Timer_Interrupt_Priority Timer_4_Interrupt_Priority
#define MEASURE_Timer_Interrupt_Subpriority Timer_4_Interrupt_Subpriority
#define MEASURE_Timer_Interrupt_Enable Timer_4_Interrupt_Enable
#define MEASURE_Timer_Vector _TIMER_4_VECTOR

#define DRIVE_Timer_Interrupt_Flag Timer_3_Interrupt_Flag
#define DRIVE_Timer_Interrupt_Priority Timer_3_Interrupt_Priority
#define DRIVE_Timer_Interrupt_Subpriority Timer_3_Interrupt_Subpriority
#define DRIVE_Timer_Interrupt_Enable Timer_3_Interrupt_Enable
#define DRIVE_Timer_Vector _TIMER_3_VECTOR

#define DEPLOY_Timer_Interrupt_Flag Timer_5_Interrupt_Flag
#define DEPLOY_Timer_Interrupt_Priority Timer_5_Interrupt_Priority
#define DEPLOY_Timer_Interrupt_Subpriority Timer_5_Interrupt_Subpriority
#define DEPLOY_Timer_Interrupt_Enable Timer_5_Interrupt_Enable
#define DEPLOY_Timer_Vector _TIMER_5_VECTOR

//Interrupt(PWM_Timer, TIMER_2, Overflow,5,1,enabled) {
//// Test_Timer_Count = 0; // reset timer
//    //MOTOR_B2 = !MOTOR_B2;
//    PWM_Timer_Interrupt_Flag = false; // clear flag
//
//}
//    
//Interrupt(MEASURE_Timer, TIMER_4,Overflow,6,1,enabled) {
//    if (MEASURE_TIMER_CONDITION==1) {
////        LORA_nRESET = high;
//        MEASURE_Timer_On = false;
//    }
//    else if (MEASURE_TIMER_CONDITION == 2) {
//            coast();
//            MEASURE_Timer_On = false;
//    }
//    else if (MEASURE_TIMER_CONDITION == 3) {
//        driveSetup();
//    }
//    else if (MEASURE_TIMER_CONDITION == 4) {
//        OC5RS = 1872;
//        MEASURE_Timer_On = false;
//    }
//    MEASURE_Timer_Interrupt_Flag = false;
//}
//Interrupt(DRIVE_Timer, TIMER_3,Overflow,6,1,enabled) {
//       in_flight_mode = in_flight_mode + 1;
//       if ((in_flight_mode%2) == 0){
//           OC5RS = 1872;
//       }
//       else {
//           OC5RS = 1807;
//       }
//       DRIVE_Timer_Interrupt_Flag = false;
//}
//Interrupt(DEPLOY_Timer, TIMER_5,Overflow,6,1,enabled) {
//    SLOW_MOTOR = !SLOW_MOTOR;
//    DEPLOY_Timer_Period = DRIVE_PERIOD - DEPLOY_Timer_Period;    
//    switch (SWITCH_MODE) {
//        case DRIVE_FORWARD:
//            driveForward();
//            break;
//        case DRIVE_BACKWARD:
//            driveBackward();
//            break;
//        case TURN_LEFT:
//            turnLeft();
//            break;
//        case TURN_RIGHT:
//            turnRight();
//            break;
//        case COAST:
//            coast();
//            break;
//    }    
//    DEPLOY_Timer_Interrupt_Flag = false;
//    
//}

//#include "interrupt_config_Timer.h"
//#include "interrupt_config_Timer_Test.h"
#include "interrupt_config_UART.h"
//#include "interrupt_config_UART_PC.h"
#include "interrupt_config_UART_LoRa.h"
//#include "interrupt_config_UART_GPS.h"
//#include "interrupt_config_UART_MUX.h"
#include "interrupt_config_Change_Notification.h"

Configure(LoRa_UART, Change_Notification)
#endif	/* INTERRUPT_CONFIG_H */
