/* 
 * File:   interrupt_config_change_notification.h
 * Author: cstgeo
 *
 * Created on February 11, 2018, 2:43 PM
 */

#ifndef INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H
#define	INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H
#include "config.h"

#define Change_Notification_Vector _CHANGE_NOTICE_VECTOR
#define Change_Notification_Interrupt_Flag IFS1bits.CNIF
#define Change_Notification_Interrupt_Priority IPC6bits.CNIP
#define Change_Notification_Interrupt_Subpriority IPC6bits.CNIS
#define Change_Notification_Interrupt_Enable IEC1bits.CNIE
#define Change_Notification_On CNCONbits.ON

typedef union {
  struct {
    unsigned _ALT_INT2:1; // CNEN0
    unsigned _ALT_INT1:1; // CNEN1
    unsigned _PGED1:1; // CNEN2
    unsigned _PGEC1:1; // CNEN3
    unsigned _BLE_nCS1:1; // CNEN4
    unsigned _BLE_nCS2:1; // CNEN5
    unsigned _BLE_nCS3:1; // CNEN6
    unsigned _V_BUSON:1; // CNEN7
    unsigned _U5TX:1; // CNEN8
    unsigned _SDA4:1; // CNEN9
    unsigned _SCL4:1; // CNEN10
    unsigned _U5RX:1; // CNEN11
    unsigned _GPS_3DFIX:1; // CNEN12
    unsigned _GAM_INT_M:1; // CNEN13
    unsigned _GAM_INT1_AG:1; // CNEN14
    unsigned _GAM_INT2_AG:1; // CNEN15
    unsigned _GAM_DRDY:1; // CNEN16
    unsigned _U2RX:1; // CNEN17
    unsigned _U2TX:1; // CNEN18
  };
  struct {
    unsigned w:32;
  };
} __CN;
extern volatile __CN CN __asm__ ("CNEN") __attribute__((section("sfrs"), address(0xBF8861D0)));

//#define Interrupt_Settings(X) X##_Interrupt_Settings()
//#define _Configure(X) Interrupt_Settings(X);
//#define Configure(...)\
// void configure_interrupts() {\
//  printf("\n\rvoid configure_interrupts()");\
//  Global_Interrupts_Enabled;\
//  Multivector_Interrupts_Enable = enabled;\
//  FOR_EACH(_Configure,__VA_ARGS__)\
// }

#define _Paste(X) X;
#define CN_Interrupt(object, full_name, event_type, priority, subpriority, is_enabled, is_on,...)\
 void Interrupt_Settings(object) {\
  /*interrupt object##_##event_type##_Event autoIPL(priority) vector VECTOR(object);*/\
  object##_Interrupt_Priority = priority; /* Sets the priority for the interrupt */\
  object##_Interrupt_Subpriority = subpriority; /* Sets the subpriority for the interrupt */\
  object##_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Interrupt_Enable = is_enabled; /* Sets the enable bit for the interrupt */\
  object##_On = is_on; /* Sets the control bit on */\
  FOR_EACH(_Paste,__VA_ARGS__)\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()

#define _CN_Enable(X) CNENbits.CNEN##X = 1;
#define CN_Enable(...) FOR_EACH(_CN_Enable,__VA_ARGS__)
#define _CNPU_Enable(X) CNPUEbits.CNPUE##X = 1;
#define CNPU_Enable(...) FOR_EACH(_CNPU_Enable,__VA_ARGS__)
#define _CN_Disable(X) CNENbits.CNEN##X = 0;
#define CN_Disable(...) FOR_EACH(_CN_Disable,__VA_ARGS__)
#define _CNPU_Disable(X) CNPUEbits.CNPUE##X = 0;
#define CNPU_Disable(...) FOR_EACH(_CNPU_Disable,__VA_ARGS__)
#define _Enable(X) X = enabled;
#define Enable(...) FOR_EACH(_Enable,__VA_ARGS__)
#define _Disable(X) X = disabled;
#define Disable(...) FOR_EACH(_Disable,__VA_ARGS__)


// Change Notification


CN_Interrupt(Change_Notification,CHANGE_NOTICE,Interrupt,6,1,enabled,true,
    CN._ALT_INT2 = disabled, // CNEN0
    CN._ALT_INT1 = disabled, // CNEN1
    CN._PGED1 = disabled, // CNEN2
    CN._PGEC1 = disabled, // CNEN3
    CN._BLE_nCS1 = enabled, // CNEN4
    CN._BLE_nCS2 = enabled, // CNEN5
    CN._BLE_nCS3 = enabled, // CNEN6
    CN._V_BUSON = disabled, // CNEN7
    CN._U5TX = disabled, // CNEN8
    CN._SDA4 = disabled, // CNEN9
    CN._SCL4 = disabled, // CNEN10
    CN._U5RX = disabled, // CNEN11
    CN._GPS_3DFIX = disabled, // CNEN12
    CN._GAM_INT_M = disabled, // CNEN13
    CN._GAM_INT1_AG = disabled, // CNEN14
    CN._GAM_INT2_AG = disabled, // CNEN15
    CN._GAM_DRDY = disabled, // CNEN16
    CN._U2RX = disabled, // CNEN17
    CN._U2TX = disabled // CNEN18
)
{
 // Check interrupt pins here.
 printf("\n\r" "void Change_Notification_Interrupt_Event()" "\n\r");
 
 if (CN._BLE_nCS1 && BLE_nCS1_IO && BLE_nCS1_R) {
  printf("\tBLE_nCS1" "\n\r");
  CN._BLE_nCS1 = false;
  BLE_nCS1_IO = output;
  BLE_nCS1 = 1;
  
  // Do stuff here that pressing BLE_nCS1 is supposed to activate
  
 } else
  if (CN._BLE_nCS2 && BLE_nCS2_IO && BLE_nCS2_R) {
  printf("\tBLE_nCS2" "\n\r");
  CN._BLE_nCS2 = false;
  BLE_nCS2_IO = output;
  BLE_nCS2 = 1;
  
  // Do stuff here that pressing BLE_nCS2 is supposed to activate
  
 } else
  if (CN._BLE_nCS3 && BLE_nCS3_IO && BLE_nCS3_R) {
  printf("\tBLE_nCS3" "\n\r");
  CN._BLE_nCS3 = false;
  BLE_nCS3_IO = output;
  BLE_nCS3 = 1;
  
  // Do stuff here that pressing BLE_nCS3 is supposed to activate
  
 } else {
  printf("\t[!] - An unexpected CNI event occurred" "\n\r");
 }
 
 // Clear Flag after pin check.
 Change_Notification_Interrupt_Flag = false;
}

#endif	/* INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H */

