/* File:   uart_config.h
 * Author: cstgeorg
 */

#ifndef UART_CONFIG_H
#define	UART_CONFIG_H
#include "config.h"



// UART Definitions
#define baudPC 57600
#define baudGPS 9600
#define baudLoRa 57600
#define baudBLE 115200
#define BR1 baudPC
#define BR2 baudLoRa
#define BR6 baudGPS
#define BRG1 FPB/4/BR1 - 1
#define BRG2 FPB/4/BR2 - 1
#define BRG6 FPB/4/BR6 - 1
#define GPS_SetBaudRateCommand "$PMTK251,115200*27\r\n"

//#define printfToUART(x) __XC_UART = (x == 2 ?  2 : 1) /* directs output of printf function through UARTx */ 
//
//#define printfToPC printfToUART(1)
//#define print(String,...) if(__XC_UART != 1) {unsigned char saved_uart_state = __XC_UART; printfToPC; printf(String,##__VA_ARGS__); __XC_UART = saved_uart_state;}\
//                          else {printf(String,##__VA_ARGS__);}
//#define printfToLORA print("Printing to LoRa\r\n"); printfToUART(2)
//#define printfToMUX(x) print("Printing to MUX(%i)\r\n",x); muxToInput(x); printfToUART(6)
//#define printfToBLE1 print("Printing to BLE1\r\n"); muxToBLE1; printfToUART(6)
//#define printfToBLE2 print("Printing to BLE2\r\n"); muxToBLE2; printfToUART(6)
//#define printfToBLE3 print("Printing to BLE3\r\n"); muxToBLE3; printfToUART(6)
//#define printfToGPS print("Printing to GPS\r\n"); muxToGPS; printfToUART(6)

#define printUARTsettings(x) printf("\n\r" "UART " #x " Setup:"\
                                   "\n\r\t" "BRG:\t%i"\
                                   "\n\r\t" "BR:\t%i bps"\
                                   "\n\r\t" "SCLK:\t%i MHz"\
                                   "\n\r\t" "PBF:\t%i MHz", U##x##BRG, BR##x, SCLK/1000000, FPB/1000000)



int u3rx_count = 0;
unsigned char getU3()
{
 while(!U3STAbits.URXDA);
// u3rx_count++;
 return U3RXREG;
}
void putU3(unsigned char c)
{
 while(U3STAbits.UTXBF);
 U3TXREG = c;
}

int u2rx_count = 0;
unsigned char getU2()
{
 while(!U2STAbits.URXDA){
 }
// u2rx_count++;
 return U2RXREG;
}
void putU2(unsigned char c)
{
    
 while(U2STAbits.UTXBF);
 U2TXREG = c;
}

int u5rx_count = 0;
unsigned char getU5()
{
 while(!U5STAbits.URXDA);
// u5rx_count++;
 return U5RXREG;
}
void putU5(unsigned char c)
{
 while(U5STAbits.UTXBF);
 U5TXREG = c;
}


/**
 * get a character from uart6 waiting until data avalable
 * @return character received
 */
void tputc(unsigned char val); // prototype

#define buffer_full U5STAbits.UTXBF  // buffer full
#define data_avail U5STAbits.URXDA   // data avail
unsigned char tgetc(void)
{
  unsigned char val;
  while (!data_avail); // wait for character to appear
  val = U5RXREG; // get character
  tputc(val); // echo
  return val;
}

void tputc(unsigned char val)
{
 switch (__XC_UART)
 {
  case 1:
   while (U1STAbits.UTXBF); // Wait till transmission is complete
   U1TXREG = val;
   break;
  case 2:
   while (U2STAbits.UTXBF); // Wait till transmission is complete
   U2TXREG = val;
   break;
  case 3:
   while (U3STAbits.UTXBF); // Wait till transmission is complete
   U3TXREG = val;
   break;
  case 4:
   while (U4STAbits.UTXBF); // Wait till transmission is complete
   U4TXREG = val;
   break;
  case 5:
   while (U5STAbits.UTXBF); // Wait till transmission is complete
   U5TXREG = val;
   break;
  case 6:
   while (U6STAbits.UTXBF); // Wait till transmission is complete
   U6TXREG = val;
   break;
  default: exit(1);
 }
}

/* helper function to make printf use tputc */
void _mon_putc(char c)
{
  tputc(c);
}
void configure_uart() {
  /* Setup UART 1 */
    __XC_UART = 3;
		  // Enable UART
		  U3MODEbits.ON = 0; // be sure it is off
          U3MODEbits.BRGH = 1; // high speed
          U3MODEbits.PDSEL = 0; // 8 bit no parity
          U3MODEbits.STSEL = 0; // one stop bit
          U3BRG = BRG1;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
          U3STAbits.UTXEN = 1; // transmit enable
          U3STAbits.URXEN = 1; // receice enable
          U3MODEbits.ON = 1; // turn it on
		  
 /* Setup UART 2 */
		  // Enable UART
		  U2MODEbits.ON = 0; // be sure it is off
          U2MODEbits.BRGH = 1; // high speed
          U2MODEbits.PDSEL = 0; // 8 bit no parity
          U2MODEbits.STSEL = 0; // one stop bit
          U2BRG = BRG2;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
          U2STAbits.UTXEN = 1; // transmit enable
          U2STAbits.URXEN = 1; // receice enable
          U2MODEbits.ON = 1; // turn it on
		  
 /* Setup UART 6 */
		  // Enable UART
          U5MODEbits.ON = 0; // be sure it is off
          U5MODEbits.BRGH = 1; // high speed
          U5MODEbits.PDSEL = 0; // 8 bit no parity
          U5MODEbits.STSEL = 0; // one stop bit
          U5BRG = BRG6;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
          U5STAbits.UTXEN = 1; // transmit enable
          U5STAbits.URXEN = 1; // receive enable
          U5MODEbits.ON = 1; // turn it on
          printf("Configure UART Complete \n\r");   
}

#endif	/* UART_CONFIG_H */

